def hex_dump(data, end_offset):
  file_offset = end_offset - len(data)
  offset = 0

  while offset < len(data):
    print_offset = file_offset + offset if end_offset != 0 else 0
    
    line = f'{print_offset:08X}   '
    
    hex_part = []
    for i in range(16):
      if offset + i < len(data):
        hex_part.append(f'{data[offset + i]:02X}')
      else:
        hex_part.append('  ')
      
      if i == 7:
        hex_part.append('')
    
    line += ' '.join(hex_part)
    line += '  '
    
    ascii_part = ''
    for i in range(16):
      if offset + i < len(data):
        byte = data[offset + i]
        if 32 <= byte <= 126:
          ascii_part += chr(byte)
        else:
          ascii_part += '.'
      else:
        ascii_part += ' '
    
    line += ascii_part
    print(line)
    
    offset += 16



import sys
import ctypes
from ctypes import c_void_p, c_uint, c_int, POINTER, byref
from enum import IntEnum
from threatcheck.scanners.base import Scanner
from threatcheck.console import Console

class AmsiResult(IntEnum):
  AMSI_RESULT_CLEAN = 0
  AMSI_RESULT_NOT_DETECTED = 1
  AMSI_RESULT_DETECTED = 32768

class AmsiScanner(Scanner):
  def __init__(self, file_bytes=None, debug=False,
      app_name='PowerShell_C:\\Windows\\System32\\WindowsPowerShell\\v1.0\\powershell.exe_5.1.22621.2506'):
    super().__init__(file_bytes=file_bytes, debug=debug)
    
    self.amsi_context = None
    self.amsi_session = None
    
    if sys.platform != 'win32':
      raise RuntimeError(
          'Platform not supported: AMSI scanner requires Windows')
    
    try:
      self.amsi = ctypes.windll.amsi
    except Exception as e:
      raise RuntimeError(f'AMSI not available on this system: {e}')
    
    self._initialize_amsi(app_name)

    if not self._real_time_protection_enabled:
      raise RuntimeError('AMSI engine requires real-time protection enabled')

  @property
  def _real_time_protection_enabled(self):
    """Check if AMSI real-time protection is enabled"""
    amsi_uid = '7e72c3ce-861b-4339-8740-0ac1484c1386'
    sample = f'Invoke-Expression \'AMSI Test Sample: {amsi_uid}\''.encode()
    result = self._scan_buffer(sample, c_void_p())
    return result == AmsiResult.AMSI_RESULT_DETECTED

  def _initialize_amsi(self, app_name):
    self.amsi_context = c_void_p()
    self.amsi_session = c_void_p()
    
    result = self.amsi.AmsiInitialize(
        ctypes.c_wchar_p(app_name),
        byref(self.amsi_context)
    )
    
    if result != 0:
      raise RuntimeError(f'AmsiInitialize failed with code {result}')
    
    result = self.amsi.AmsiOpenSession(
        self.amsi_context,
        byref(self.amsi_session)
    )
    
    if result != 0:
      self.cleanup()
      raise RuntimeError(f'AmsiOpenSession failed with code {result}')
  
  def _scan_bytes(self, data):
    """Scan a data split for threats"""
    status = self._scan_buffer(data)
    return status == AmsiResult.AMSI_RESULT_DETECTED

  def _scan_buffer(self, buffer, session=None):
    if session is None:
      session = self.amsi_session
    
    result = c_int()
    
    buffer_array = (ctypes.c_ubyte * len(buffer)).from_buffer_copy(buffer)
    
    ret = self.amsi.AmsiScanBuffer(
        self.amsi_context,
        buffer_array,
        c_uint(len(buffer)),
        ctypes.c_wchar_p('test.ps1'),
        session,
        byref(result)
    )
    
    if ret != 0:
      Console.write_error(f'AmsiScanBuffer failed with code {ret}')
      return AmsiResult.AMSI_RESULT_CLEAN
    
    return AmsiResult(result.value)

  def cleanup(self):
    if self.amsi_session and self.amsi_context:
      try:
        self.amsi.AmsiCloseSession(self.amsi_context, self.amsi_session)
      except:
        pass
    
    if self.amsi_context:
      try:
        self.amsi.AmsiUninitialize(self.amsi_context)
      except:
        pass

  def __enter__(self):
    return self

  def __exit__(self, exc_type, exc_val, exc_tb):
    self.cleanup()
    return False

  def __del__(self):
    self.cleanup()



import sys
import argparse
import requests
from pathlib import Path
from dataclasses import dataclass
from typing import Optional, List
from enum import Enum
from threatcheck import __version__
from threatcheck.console import Console
from threatcheck.scanners.defender import DefenderScanner
from threatcheck.scanners.amsi import AmsiScanner

class ScanStatus(Enum):
  SUCCESS = 'success'
  NO_THREAT = 'no_threat'
  THREAT_FOUND = 'threat_found'
  ERROR = 'error'
  SKIPPED = 'skipped'

@dataclass
class ScanResult:
  file_path: Path
  status: ScanStatus
  error_message: Optional[str] = None
  file_size: Optional[int] = None

  @property
  def success(self) -> bool:
    return self.status in (ScanStatus.SUCCESS,
                           ScanStatus.NO_THREAT,
                           ScanStatus.THREAT_FOUND)

  @property
  def filename(self) -> str:
    return self.file_path.name

def download_file_bytes(url):
  try:
    response = requests.get(url, timeout=30)
    response.raise_for_status()
    return response.content
  except requests.RequestException as e:
    raise RuntimeError(f'Could not connect to URL: {e}')

def list_files_in_directory(directory: Path) -> List[Path]:
  if not directory.exists():
    Console.write_error('Directory not found')
    sys.exit(1)
  if not directory.is_dir():
    Console.write_error('Path is not a directory')
    sys.exit(1)
  files = [f for f in directory.iterdir() if f.is_file()]
  if not files:
    Console.write_error('No files found in directory')
    sys.exit(1)
  return files

def read_file_content(file_path: Path) -> bytes:
  with open(file_path, 'rb') as f:
    return f.read()

def initialize_scanner(engine: str,
                       debug: bool,
                       file_bytes: bytes = None):
  if engine == 'defender':
    return DefenderScanner(debug=debug, file_bytes=file_bytes)
  elif engine == 'amsi':
    return AmsiScanner(debug=debug, file_bytes=file_bytes)
  else:
    raise ValueError(f'Unknown engine: {engine}')

def scan_file_bytes(file_path: Path,
                    file_bytes: bytes,
                    engine: str,
                    debug: bool) -> ScanResult:
  try:
    scanner = initialize_scanner(engine, debug, file_bytes=file_bytes)
    scanner.analyze()
    if scanner.malicious:
      status = ScanStatus.THREAT_FOUND
    else:
      status = ScanStatus.NO_THREAT
    return ScanResult(file_path=file_path,
                      status=status,
                      file_size=len(file_bytes))
  except Exception as e:
    Console.write_error(f'Error scanning {file_path.name}: {e}')
    return ScanResult(
        file_path=file_path,
        status=ScanStatus.ERROR,
        error_message=str(e))

def process_files(file_list: List[Path], args) -> List[ScanResult]:
  results = []

  Console.write_output(f'Found {len(file_list)} file(s) to scan')

  for file_path in file_list:
    Console.write_output(f'Scanning file: {file_path.name}')
    content = read_file_content(file_path)
    result = scan_file_bytes(file_path,
                             content,
                             args.engine.lower(),
                             args.debug)
    results.append(result)

  return results

def print_summary(results: List[ScanResult]):
  success = sum(1 for r in results if r.success)
  errors = len(results) - success
  threats = sum(1 for r in results if r.status == ScanStatus.THREAT_FOUND)
  Console.write_output('\n')
  Console.write_output('-' * 60)
  Console.write_output(f'Scan complete: {success} successful, {errors} errors')
  if threats:
    Console.write_threat(f'Found {threats} threats')
    for result in results:
      if result.status == ScanStatus.THREAT_FOUND:
        Console.write_threat(f'  - {result.file_path}')
  else:
    Console.write_output('No threats found')
  

def parse_arguments():
  parser = argparse.ArgumentParser(
      description='Identify AV signatures in files',
      prog='threatcheck')
  parser.add_argument(
      '-e', '--engine',
      type=str.lower,
      default='defender',
      choices=['defender', 'amsi'],
      help='Scanning engine (default: defender)')
  parser.add_argument(
      '-f', '--file',
      help='Analyze a file on disk')
  parser.add_argument(
      '-u', '--url',
      help='Analyze a file from a URL')
  parser.add_argument(
      '-d', '--directory',
      help='Analyze all files in a directory')
  parser.add_argument(
      '--debug',
      action='store_true',
      help='Enable debug output')
  parser.add_argument(
      '--version',
      action='version',
      version=f'%(prog)s {__version__}')

  return parser.parse_args()


def main():
  args = parse_arguments()

  if args.directory:
    file_list = list_files_in_directory(Path(args.directory))
    results = process_files(file_list, args)
  elif args.file:
    file_list = [Path(args.file)]
    results = process_files(file_list, args)
  elif args.url:
    file_content = download_file_bytes(args.url)
    results = [scan_file_bytes(Path(args.url),
                               file_content,
                               args.engine.lower(),
                               args.debug)]
  else:
    Console.write_error(
        'Specify either -d, -f or -u as a source. Type --help for more.')
    sys.exit(1)

  print_summary(results)


import os
import sys
import subprocess
from pathlib import Path
from enum import Enum

from threatcheck.scanners.base import Scanner
from threatcheck.console import Console

class ScanResult(Enum):
  NO_THREAT_FOUND = 0
  THREAT_FOUND = 2
  FILE_NOT_FOUND = 3
  TIMEOUT = 4
  ERROR = 5

class DefenderScanResult:
  def __init__(self):
    self.result = None
    self.signature = None

class DefenderScanner(Scanner):
  def __init__(self, file_bytes=None, debug=False):
    super().__init__(file_bytes=file_bytes, debug=debug)
    
    if sys.platform != 'win32':
      raise RuntimeError(
          'Platform not supported: Defender scanner requires Windows')
    
    self.mpcmdrun_path = Path(r'C:\Program Files\Windows Defender\MpCmdRun.exe')
    if not self.mpcmdrun_path.exists():
      raise FileNotFoundError(
          f'MpCmdRun.exe not found at {self.mpcmdrun_path}')
  
  def _scan_bytes(self, data):
    """Scan a data split for threats"""
    # Defender needs to scan from disk, not memory.
    testfile_path = os.path.join(self.temp_dir, 'file.exe')
    with open(testfile_path, 'wb') as f:
      f.write(data)
    
    status = self._scan_file(testfile_path)
    return status.result == ScanResult.THREAT_FOUND

  def _scan_file(self, file_path, get_sig=False):
    result = DefenderScanResult()
    
    if not os.path.exists(file_path):
      result.result = ScanResult.FILE_NOT_FOUND
      return result
    
    try:
      cmd = [
        str(self.mpcmdrun_path),
        '-Scan',
        '-ScanType', '3',
        '-File', file_path,
        '-DisableRemediation',
        '-Trace',
        '-Level', '0x10'
      ]
      
      process = subprocess.Popen(
          cmd,
          stdout=subprocess.PIPE,
          stderr=subprocess.PIPE,
          creationflags=subprocess.CREATE_NO_WINDOW if sys.platform == 'win32'
              else 0
      )
      
      try:
        stdout, stderr = process.communicate(timeout=30)
      except subprocess.TimeoutExpired:
        process.kill()
        result.result = ScanResult.TIMEOUT
        return result
      
      if get_sig:
        output = stdout.decode('utf-8', errors='ignore')
        for line in output.split('\n'):
          if 'Threat  ' in line:
            parts = line.split(' ')
            if len(parts) > 19:
              result.signature = parts[19]
            break
      
      if process.returncode == 0:
        result.result = ScanResult.NO_THREAT_FOUND
      elif process.returncode == 2:
        result.result = ScanResult.THREAT_FOUND
      else:
        result.result = ScanResult.ERROR
      
      return result
    
    except Exception as e:
      Console.write_error(f'Error scanning file: {e}')
      result.result = ScanResult.ERROR
      return result




__version__ = '0.1.0'
__author__ = 'Werebug security@werebug.com'
__license__ = 'GPL-3.0'

from threatcheck.scanners.defender import DefenderScanner
from threatcheck.scanners.amsi import AmsiScanner

__all__ = ['DefenderScanner', 'AmsiScanner']



import sys
from colorama import Fore, Style, init

init(autoreset=True)

class Console:
  @staticmethod
  def write_output(message):
    print(f'{Fore.GREEN}[+] {message}{Style.RESET_ALL}')

  @staticmethod
  def write_error(message):
    print(f'{Fore.RED}[x] {message}{Style.RESET_ALL}', file=sys.stderr)

  @staticmethod
  def write_debug(message):
    print(f'{Fore.YELLOW}[*] {message}{Style.RESET_ALL}')

  @staticmethod
  def write_threat(message):
    print(f'{Fore.RED}[!] {message}{Style.RESET_ALL}')



from threatcheck.scanners.base import Scanner
from threatcheck.scanners.defender import DefenderScanner
from threatcheck.scanners.amsi import AmsiScanner

__all__ = ['Scanner', 'DefenderScanner', 'AmsiScanner']



import os
import tempfile
from abc import ABC, abstractmethod
from pathlib import Path

from threatcheck.console import Console
from threatcheck.helpers import hex_dump

class Scanner(ABC):
  def __init__(self, file_bytes=None, debug=False):
    if not file_bytes:
      raise ValueError('file_bytes must be provided')
    
    self.file_bytes = file_bytes
    self.debug = debug
    self.malicious = False
    self.complete = False
    self.temp_dir = None

  def analyze(self):
    """Starts data analysis on either process memory or file bytes."""
    self.temp_dir = tempfile.mkdtemp()
    try:
      self._start_file_scan()
    finally:
      self._cleanup_temp_files()
  
  def _cleanup_temp_files(self):
    """Removes temporary files created during analysis."""
    for file in Path(self.temp_dir).iterdir():
      try:
        os.remove(file)
      except:
        Console.write_error(f'Failed to remove temporary file: {file}')
    
    try:
      os.rmdir(self.temp_dir)
    except:
      Console.write_error(
          f'Failed to remove temporary directory: {self.temp_dir}')

  def _half_splitter(self, original_array, last_good):
    """Splits the array in half, keeping the first half.
    Called when a threat is found to reduce the data size."""
    split_size = (len(original_array) - last_good) // 2 + last_good
    split_array = original_array[:split_size]

    if len(original_array) == split_size + 1:
      msg = f'Identified end of bad bytes at offset 0x{len(original_array):X}'
      Console.write_threat(msg)

      offending_size = min(len(original_array), 256)
      offending_bytes = original_array[-offending_size:]

      hex_dump(offending_bytes, len(original_array))
      self.complete = True

    return split_array

  def _overshot(self, original_array, split_array_size):
    """Called when no threat is found to increase the data size."""
    new_size = (len(original_array) - split_array_size) // 2 + split_array_size

    if new_size == len(original_array) - 1:
      self.complete = True

      if self.malicious:
        Console.write_error('File is malicious, but couldn\'t identify bad bytes')

    return original_array[:new_size]
  
  def _binary_split_loop(self, initial_scan_result):
    """Common binary splitting logic.
    
    Searches the exact bytes where the signature ends by keeping track of the
    last known good bytes and splitting the remaining bytes in half.
    """
    if not initial_scan_result:
      Console.write_output('No threat found!')
      return
    
    self.malicious = True
    
    Console.write_output(f'Target file size: {len(self.file_bytes)} bytes')
    
    split_array = self.file_bytes[:len(self.file_bytes) // 2]
    last_good = 0
    
    while not self.complete:
      if self.debug:
        Console.write_debug(f'Testing {len(split_array)} bytes')
      
      detection_result = self._scan_bytes(split_array)
      
      if detection_result:
        if self.debug:
          Console.write_debug('Threat found, splitting')
        
        split_array = self._half_splitter(split_array, last_good)
      else:
        if self.debug:
          Console.write_debug('No threat found, increasing size')
        
        last_good = len(split_array)
        split_array = self._overshot(self.file_bytes, len(split_array))
  
  def _start_file_scan(self):
    """Analyze file bytes with binary splitting"""
    initial_threat = self._scan_bytes(self.file_bytes)
    
    if self.debug:
      Console.write_debug(f'Status value: {initial_threat}')
    
    self._binary_split_loop(initial_threat)
  
  @abstractmethod
  def _scan_bytes(self, data):
    """Subclasses implement specific scan methods.
    
    Args:
        data: The bytes to scan
        
    Returns:
        bool: True if threat detected, False otherwise
    """
    pass
